--timer to update
--top half of GUI
--

EMPTY_FX_COLUMN = 14
 
require "_1a_upper_panel_functions"
require "_1_upper_panel_gui_notifiers"
require "_upper_panel_timer"

---------------------------------------------Sample Commands---------------------------------------------------------
--[[
These commands only apply to samples and will have no effect on the Plugin or MIDI components of an Instrument. When used in the Master track, the effect will be applied to all tracks.

    1 -Axy - Set arpeggio, x/y = first/second note offset in semitones. Using 0 for x or y will use the original note.
    2 -Uxx - Slide pitch up by xx 1/16ths of a semitone (01 is 1/16th of a semitone, 08 is half a semitone, 10 is a whole semitone).
    3 -Dxx - Slide pitch down by xx 1/16ths of a semitone (01 is 1/16th of a semitone, 08 is half a semitone, 10 is a whole semitone).
    4 -Gxx - Glide towards given note by xx 1/16ths of a semitone (01 is 1/16th of a semitone, 08 is half a semitone, 10 is a whole semitone). A value of FF will make the slide instant.
    5 -Ixx - Fade volume in by xx volume units (-I01 inserted 256 times will slide from 0 to full volume, -I7F inserted twice will do the same much faster).
    6 -Oxx - Fade volume out by xx volume units.
    7 -Txy - Set tremolo (regular volume variation), x = speed, y = depth.
    8 -Cxy - Cut volume to x after y ticks (x = volume factor: 0=0%, F=100%).
    9 -Sxx - Trigger sample slice number xx or offset xx.
    10-Bxx - Play sample backwards (xx = 00) or forwards (xx = 01).
    11-Vxy - Set vibrato (regular pitch variation), x = speed, y = depth.
    12-Nxy - Set auto pan (regular pan variation), x = speed, y = depth.
    13-Exx - Set the position of all active Envelope, AHDSR and Fader Modulation devices to offset xx. 
--]]
-----------------------------------------------------------------------------------------------------------------------




--table for beat marker labels in transport buttons
beat_markers = {}
---------------------------------
function calculate_beat_markers()
---------------------------------
  local beat_markers = {}
  local lpb = renoise.song().transport.lpb
  --loop to fill table
  for i = 1,tool_range do
    if (i - 1) % lpb == 0 then 
      beat_markers[i] = "|" --beat marker
    else
      beat_markers[i] = "" --off beat
    end
  end
   return beat_markers
end


-------------------------
--get renoise track names
-------------------------
function get_renoise_track_names()
  local song = renoise.song()
  local track_name_table = {}
  for track = 1,#song.tracks do
    --if song.tracks[track].type == renoise.Track.TRACK_TYPE_SEQUENCER then
      table.insert(track_name_table,song.tracks[track].name)
   -- end
  end
  return track_name_table
end 


table_of_sample_effects = {}
table_of_sample_effects.name_strings = nil
table_of_sample_effects.number_strings = nil

table_of_sample_effects.name_strings = {

 "Arp",--...0AXY", 1
 "Pitch up",--...0UXX", 2
 "Pitch down",--...0DXX", 3
 "Glide",--...0GXX", 4
 "Fade In",--...0IXX",  5
 "Fade Out",--...0OXX", 6
 "Tremolo",--...0TXY", 7
 "Cut",--...0CXY", 8
 "Samp. Offset",--...0SXX", 9
 "Backwards",--...0BXX", 10
 "Vibrato",--...0VXY", 11
 "Auto Pan",--...0NXY",  12
 "Env. offset",--...0EXX", 13
 "---" --14 empty
}
table_of_sample_effects.number_strings = {

 "0A",--...0AXY", 1 Arp
 "0U",--...0UXX", 2 Pitch up
 "0D",--...0DXX", 3 Pitch Down
 "0G",--...0GXX", 4 "Glide"
 "0I",--...0IXX", 5 Fade in
 "0O",--...0OXX", 6 Fade out
 "0T",--...0TXY", 7 Tremolo
 "0C",--...0CXY", 8 Cut
 "0S",--...0SXX", 9 Samp Offset
 "0B",--...0BXX", 10 Backwards
 "0V",--...0VXY", 11 Vibrato
 "0N",--...0NXY", 12 Autopan
 "0E",--...0EXX", 13 Env. Offset
 "---"          --14 empty
}

--[[-missing from online manual

Q


--]]
-------------------------------------------
function get_fx_column_targets_for_popups()
-------------------------------------------
  local song = renoise.song()
  --1)--loop all fx columns of selected track to find any present fx data
  --set the popups to the first fx type found there
  for j = 1,8 do
    --if popup shows no effect found it`s default string [---]
    if vb.views["popup col "..j].value == #table_of_sample_effects.name_strings then --works

      --loop lines in fx column
      local found_flag = false
      for line = 1, #song.selected_pattern_track.lines do
        if found_flag == true then
          break
        end
        local fx = song.selected_pattern_track:line(line):effect_column(j)
        --loop to find match with popup table
        for popup_idx = 1, #table_of_sample_effects.number_strings do
          if fx.number_string == table_of_sample_effects.number_strings[popup_idx] then --get first two characters
            vb.views["popup col "..j].value = popup_idx
            found_flag = true
            break
          end
        end
      end
    end
  end
end

------------------------------------
function show_grab_extra_buttons(val)
------------------------------------
   for i = 1,tool_range do
     vb.views["grab extra"..i].visible = val
     --reset text to empty
     vb.views["grab extra"..i].text = ""
     --reset color to grey to empty
     vb.views["grab extra"..i].color = COLOR_GREY
     --reset grab target to nil
     g_grab_extra_target_button = nil
   end
end

--transport text
--options.transport_text_flag.value = "beat markers" --can be "beat markers","pattern line numbers" or "pattern notes" 

------------------------------------
------------------------------------
function main_upper_panel()
------------------------------------
------------------------------------


  --toggle dialog so if its already open then close it.  The timer will catch
  --that it is closed and do the housekeeping like removing notifiers (and timer itself)
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    return
  end
  
  --BUG CHECK
  --in case tool crashed and notifiers weren`t removed
  if remove_notifiers() == true then
    tprint("notifiers removed on start")
  end
  
  --renoise song object
  local song = renoise.song()
  
  --get song bpm
  g_original_bpm = renoise.song().transport.bpm
  --calculate beat markers table --"|" marker spacing in transport buttons
  beat_markers = calculate_beat_markers()
  
  
  --------------------------------------
  local function notifier_show_bottom_panel()
  ------------------------------------
    --toggle view if already constructed
    if vb.views["bottom panel"] then
      if vb.views["bottom panel"].visible == true then
        vb.views["bottom panel"].visible = false
        return
      else
        vb.views["bottom panel"].visible = true
        return
      end
    end
    --else create bottom panel  
     dialog_content:add_child(show_bottom_panel())
   end

  --set globals
  -------------
  --assign global vb to viewbuilder
  vb = renoise.ViewBuilder()
  
  ----------------------------------------------
  --GUI
  ----------------------------------------------     

  

   
  
  --get renoise note names
  ---------------------------------------
  local function get_renoise_note_names()
  ---------------------------------------
    local note_name_table = {}
    local note_strings = {
    "C-", "C#", "D-", "D#", "E-", "F-", 
    "F#", "G-", "G#", "A-", "A#", "B-"
      }
    
    local counter = 1
    for octave = 1,10 do
      for note = 1,#note_strings do
        note_name_table[counter] = note_strings[note]..(octave-1) 
        counter = counter+ 1
      end
    end
    return note_name_table
  end
-------------------------------



  --get renoise note names in reverse order (otherwise higest notes occupy lowest position in the popup)
  ---------------------------------------
  local function get_renoise_note_names_reverse_order()
  ---------------------------------------
    local note_name_table = {}
    local note_strings = {
    "C-", "C#", "D-", "D#", "E-", "F-", 
    "F#", "G-", "G#", "A-", "A#", "B-"
      }
    
    local counter = 1
    --loop backwards octaves
    for octave = 10,1,-1 do
      for note = #note_strings,1,-1 do
        note_name_table[counter] = note_strings[note]..(octave-1) 
        counter = counter + 1
      end
    end

    return note_name_table
  end
  
 -- rprint(get_renoise_note_names())
  --rprint(get_renoise_note_names_reverse_order())
-------------------------------
-----------  
--build GUI
-----------



--main rows of the tool gui
--vb:rows{}
local solo_row = vb:row {margin = 4} --set default margin for each row `4`
local transport_and_sliders_row = vb:row {margin = 4}
local fx_col_buttons_popups = vb:row {margin = 4}
-------------------------------


local active_buttons = nil
local groove_sliders = nil
local note_column_select_buttons = nil

----------------------------------------------------------------------------------  
----------------------------------1)TOP ROW---------------------------------------
--[solo] [1/2 bpm] [track select] [rename track] [default note] [clear silent notes]
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
local row_one = vb:row{ 
  -- margin = 4,
   
   vb:button{
    text = "Solo",
    width = 80,
    height = 20,
    color = COLOR_GREY,
    id = "solo button",
    notifier = function() solo_button_notifier() end
   },
    
   vb:button{
    text = "1/2 BPM",
    color = COLOR_GREY,
    height = 20,
    width = 65,
    id = "bpm button",
    notifier = function() bpm_button_notifier() end
   },
   
   vb:popup{
    width = 120,
    height = 20,
    items = get_renoise_track_names(),
    value = song.selected_track_index,
    id = "track names",
    notifier = function(value) 
      --return early if it was the timer changing the value and not the user
      if timer_running == true then
        return 
      end
      --run notifier function
      track_names_popup_notifier(value)
      end
   },
                
   vb:textfield{
    id = "rename track txt",
    notifier = function(value) rename_track_textfield_notifier(value) end 
   }, 
   
   vb:text{
    text = ""
   },  --spacer
     
   vb:button{
    id = "R button",
    text = "R",--reset target note popup to C-4
    notifier = function()
      --print(g_target.default_note)
      vb.views["target note"].value = 72--49 --C-4
    end
   },
     
   vb:popup{
    id = "target note",
    items = get_renoise_note_names_reverse_order(), --get_renoise_note_names(),
    value = (120 - g_target.default_note ), --default note (saved as global)
    notifier = function(value)
      --update global, (used forsetting initial value of popup [per-session, not an options.value],
      --value has to be (120 -) as popup order is reversed from default
      g_target.default_note = 120 - value
     --if timer change popup value, don`t run 
     if timer_running == true then 
       return
     end
     set_default_note_for_entry_popup(value)
     --run timer to update gui quicker in [G] mode
     upper_panel_timer() --loop stopped by flag above
   end
  },
   
  
   
   vb:button{
    id = "grab button",
    color = COLOR_GREY,
    width = 50,
    text = "G",
    notifier = function()
      if vb.views["grab button"].color[1] == COLOR_GREY_FLAG then
        
        --show grab extra buttons, checking checkbox enabling them first
        if vb.views["grab lane enabled cb"].value == true then
          show_grab_extra_buttons(true)
        end
        vb.views["grab button"].color = GRAB_BUTTON_ENABLED_COLOR
        --disable song edit mode
        song.transport.edit_mode = false
        --enable note + velocity target
       
       -- volume_and_note_target_notifier
        note_column_target_notifier(2)  --(value,button)
    
       -- g_target.value_type needs to  == "note + volume"
       g_target_notifier_tab_notes[2]()
       
              
      else
        --hide grab extra buttons
        show_grab_extra_buttons(false)
        --turn button grey, timer takes care of the rest
        vb.views["grab button"].color = COLOR_GREY
      end
    end
     
     --  set_default_note_for_entry_popup(value) end
   },
   
   vb:checkbox{
    id = "grab lane enabled cb", 
    value = options.grab_target_lane_checkbox_enabled.value, --stored in options
    notifier = function(value)
      if timer_running == true then
        return
      end
      --update prefernce to store state
      options.grab_target_lane_checkbox_enabled.value = value
      --show lane if we are in grab mode
      if value == true then
        if vb.views["grab button"].color[1] == GRAB_BUTTON_ENABLED_COLOR_FLAG then
          show_grab_extra_buttons(true)
        end
      else --hide lane
        show_grab_extra_buttons(false)
      end
    end
    },
   
   vb:text{}, --spacer
                
   vb:button{
     text = "Clear Silent Notes bll",
     notifier = function() clear_silent_notes_button() end,
    },
    
   vb:button{
     text = "CHORD MODE",
     notifier = function()  end,
    },
    
    
         
   } --end of first row               
                      
solo_row:add_child(row_one) 
----------------------------
--end of first row

       
-------------------2)-SELECT NOTE COLUMN BUTTONS AND POPUPS----------------
--[NOTE 1] [NOTE 2] [NOTE 3]....etc
--[Volume] [pitch] [Empty]....etc
------------------------------------------------------------------------     
--view to be added into dialog_content later
local note_col_buttons_and_popups  = vb:row{
                          margin = 4
                     } 
--[[      
--table of targets to populate the popups
local table_of_note_effects = {
 "Note + Vol.",
 "Volume",
 "Pitch",
 "Delay",
 "Pan",
}
--]]

local table_of_note_effects = {
 "Pitch",
 "Note + Vol.",
 "Volume",
 "Pan",
 "Delay",
}

--loop to create for all note columns
-------------------------------------
for i = 1, 12 do --number of available fx columns in renoise
  local note_col_popup = vb:row{ 
    vb:column{
     --BUTTONS TO CHOOSE WHICH NOTE COLUMN TO TARGET
     -----------------------------------------------
     vb:button{
      text = "NOTE "..i,
      id = "Note Column Button "..i,
      height = 26,
      width = 90,
      color = COLOR_GREY,
      notifier = function()
        --return early if it was the timer changing the value and not the user
        if timer_running == true then
          return 
        end
        --run notifier function
        note_selection_buttons_notifier(i)
        --now run timer() to speed up update of sliders 
        --(better than doing inside notfier function in case it is called elsewhere and we don`t want timer fired)
        upper_panel_timer()    
        end
     },
     --POPUPS TO CHOOSE WHICH NOTE COLUMN VALE TO TARGET "VOL""PITCH"....                    <<< ---DELETE THESE
      -----------------------------------------------------------------
    --[[ vb:popup{ 
      items = table_of_note_effects,
      id = "Note popup col "..i,
      value = 1,--defalt to Vol
      width = 90,
      notifier = function(value) note_popup_selection_notifier(value,i) end,
      visible = false,
      }, --]] 
     }
    }
 --add into view note_col_buttons_and_popups
 note_col_buttons_and_popups:add_child(note_col_popup)
end --for loop
--END OF NOTE COLUMN  BUTTONS  


local note_col_target_buttons = vb:horizontal_aligner{vb:text{text = ""}} --,mode = "center"}

--loop to add buttons
for i = 1,5 do

  local button = vb:button{
    id = "note col target "..i,
    text = table_of_note_effects[i],
    width = 80,
    color = COLOR_GREY,
    notifier = function()
        --return early if it was the timer changing the value and not the user
        if timer_running == true then
          return 
        end
        --run notifier function
        note_column_target_notifier(i)
      end
   }
  note_col_target_buttons:add_child(button)
end










--row 3) updates the user on what the target is:
---------------------------------(3)-----------------------------------------------  
local target_readout_row =
  vb:horizontal_aligner{
   margin = 4,
   mode = "center",
   vb:row{
    style = "border",
    vb:text{
    id = "Selected target",
    font = "big",
    height = 25,
    width = 300,
   }
  }
 }


---------------------------------(4)-----------------------------------------------    

----------------------------------------------------------------------------------
--[Transport Buttons] [MAIN SLIDERS] [Note Textfields][OFF Buttons](red when note off)
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------   
------------------------------------
local function get_button_color(num)
------------------------------------
  if num < 5 then
    return COLOR_GREEN
  else
    return COLOR_GREY
  end
end
---------------------------------------------------------------------------------
--row 4) TRANSPORT BUTTONS + SLIDERS  [Note Textfields] [OFF Buttons](red when note off)
---------------------------------------------------------------------------------



--loop to create buttons 
for button = 1,tool_range do
  
  local panel = 
    vb:column{
      
     vb:button{ ------------transport buttons-----
      id = "transport button"..button, 
      width = SLIDER_WIDTH,
      height = 20,
      text = beat_markers[button],--can be changed for the function?
      color = get_button_color(button),
      notifier = function() transport_buttons_notifier(button) end          
     },
     
 

      vb:button{ ------------grab buttons-----
       id = "grab extra"..button, 
       width = SLIDER_WIDTH,
      height = 20,
      -- text = beat_markers[button],--can be changed for the function?
      color = COLOR_GREY,
      visible = false,
      notifier = function()
      
       --toggle this button to grey and inactive
       if vb.views["grab extra"..button].color[1] == COLOR_RED_FLAG then
         vb.views["grab extra"..button].color = COLOR_GREY
         vb.views["grab extra"..button].text = ""
         g_grab_extra_target_button = nil
         return
       end 
       
       --or enable
       if vb.views["grab button"].color[1] == GRAB_BUTTON_ENABLED_COLOR_FLAG then
         --turn buttons grey and clear text
        for i = 1,tool_range do
          vb.views["grab extra"..i].color = COLOR_GREY
          vb.views["grab extra"..i].text = ""
        end
        --get target note text
        local text = vb.views["target note"].items[vb.views["target note"].value]
        --turn this button red
        vb.views["grab extra"..button].color = COLOR_RED
        vb.views["grab extra"..button].text = text
        --update global
        g_grab_extra_target_button = button

        return
      end 
      end          
     },
     

     
 
              
     ---***********SLIDERS******************
     -----------------------------------------
     vb:column{
       id = "slider column "..button,--enables hilighting sliders on the beat
       
       vb:slider{ 
        width = SLIDER_WIDTH,
        height = 150 ,
        min = 0,
        max = VEL_SLIDER_MAX,
        id = "slider"..button,
        notifier = function(value)
          if timer_running then
            return
          end
         main_sliders_notifier(value,button) end 
       },
     },
     
     
         ---NOTE TEXTFIELDS---
     vb:textfield{ 
      text ="---",-- get_renoise_track_names()[49],
      width = SLIDER_WIDTH,
      id = "note name textfield"..button,
      --visible = false,
      notifier = function(text)
        if timer_running then
          return
        end
        note_name_textfield_notifier(text,button) end  
     },
     
  
  
     vb:button{
      id = "X button"..button,
      width = SLIDER_WIDTH,
     -- text = "x",
     -- height = 36,
      color = COLOR_GREY,
      visible = true,
      notifier = function() 
          --g_target = {}
          --g_target.column_index = 1
          --g_target.column_type = "note"
          --g_target.value_type = "volume"
          --g_target.all_note_columns = nil
          --g_target.all_fx_columns = nil
          
          --rprint(g_target)
          --print(g_target.column_index)
         
          local song = renoise.song()       
          if g_target.column_type == "note" then
  
            local note = song.selected_pattern_track:line(button + tool_offset):note_column(g_target.column_index)
            if (g_target.value_type == "pitch") and note.note_string ~= "OFF" then
              if vb.views["X button"..button].color[1] == COLOR_RED_MILD_FLAG then
                note.note_string = "---"
                note.instrument_string = ".."
              else
                note.note_string = "C-4"
              end
            elseif g_target.value_type == "note + volume" and note.note_string ~= "OFF" then
              --delete if button red (volume will be 00 in this case)
              if vb.views["X button"..button].color[1] == COLOR_RED_MILD_FLAG then
                note.note_string = "---"
                note.volume_string = ".."
                note.instrument_string = ".."
              else
                note.volume_string = "00"
              end
             -- note.instrument_string = ".."
            elseif g_target.value_type == "volume" then
              note.volume_string = ".."
            elseif g_target.value_type == "pan" then
              note.panning_string = ".."
            elseif g_target.value_type == "delay" then
              note.delay_string = ".."
            end
          elseif g_target.column_type == "effect" then
            local fx = song.selected_pattern_track:line(button + tool_offset):effect_column(g_target.column_index)
            fx:clear()
         end
         upper_panel_timer()
       end
       },

              
     vb:button{
      id = "off button"..button, --Blue when OFF / Clear Note buttons
      width = SLIDER_WIDTH,
     -- text = "o",
      color = COLOR_GREY,
      notifier = function() off_buttons_notifier(button) end

     },
     
    }
  --highlight sliders on the beat  
  --if (button - 1) % 4 == 0 then
  -- vb.views["slider column "..button].style = "panel"
   --vb.views["beat marker text"..button].text ="    >"
 -- end

  --add each iteration into transport_and_sliders_row
  transport_and_sliders_row:add_child(panel) 
end --end loop

-----------------------------------------------------
--end of row 4---------------------------------------

--create a long text spacer
---------------------------
local spacer_text_1 = " "
for i = 1,322 do
  spacer_text_1 = spacer_text_1.." "
end
--------------------------
local off_buttons_popup =
 vb:row{
   vb:text{
    text = spacer_text_1
   },
   vb:switch{
    id = "off vs x popup",
    items = {"OFF ","X "},--,"Both"},
    value = options.show_off_or_x_buttons_popup.value,-- options.show_off_or_x_buttons_popup.value,
    width = 150,
    notifier = function(value)
     --update preference
     options.show_off_or_x_buttons_popup.value = value
     if vb.views["off vs x popup"].value == 1 then
       
       for i = 1, tool_range do
         vb.views["off button"..i].visible = true 
         vb.views["X button"..i].visible = false
         
        -- vb.views["note name textfield"..i].visible = true
       end
     elseif vb.views["off vs x popup"].value == 2 then
       for i = 1, tool_range do   
         vb.views["off button"..i].visible = false 
         vb.views["X button"..i].visible = true
       -- vb.views["note name textfield"..i].visible = false
       end
      --[[
     elseif vb.views["off vs x popup"].value == 3 then
       for i = 1, tool_range do   
         vb.views["off button"..i].visible = true 
         vb.views["X button"..i].visible = true
       end --]]  
     end
   
   
   
   
   end
  }
 }




--row 5)-----------------------------------------------------------------------       
--------------------SELECT COLUMN BUTTONS AND POPUPS---------------------------
--[COL 1] [COL 2] [COL 3]....etc
--[Glide] [pitch] [Empty]....etc
-------------------------------------------------------------------------------      
      

for i = 1, song.selected_track.max_effect_columns do --number of available fx columns in renoise
  
local fx_col_popups = vb:row{  --change name here-

    vb:column{
      vb:button{
       text = "FX "..i,
       id = "Fx Column Button "..i,
       height = 26,
       width = 90,
       color = COLOR_GREY,
       notifier = function()
                   if timer_running == true then 
                      return
                   end 
                   fx_column_buttons_notifier(i)
                   --run timer to update gui quicker/ smoothly
                   upper_panel_timer()  
                   end        
      },

     vb:popup{
      items = table_of_sample_effects.name_strings,
      id = "popup col "..i,
      value = EMPTY_FX_COLUMN,--defalt to empty "---"
      width = 90,
      notifier = function(value) 
                   if timer_running == true then
                     return
                   end
                   --use same notifier as button above (both get set at the same time)
                   fx_column_buttons_notifier(i)
                  -- fx_column_popups_notifier(value,i)
                 end
     },   
    },  
   }
 
  fx_col_buttons_popups:add_child(fx_col_popups) 
end--end row 5)

  
 
   --ASSEMBLE GUI
  --------------------------------------------------------
  --------------------------------------------------------
  --dialog content: master cvb:column{}
  --will contain all of gui; passed to renoise.app():show_custom_dialog()
  dialog_content = vb:column{ margin = 6 }
  --add views
  dialog_content:add_child(solo_row)
  dialog_content:add_child(note_col_buttons_and_popups)
  dialog_content:add_child(note_col_target_buttons)
  dialog_content:add_child(target_readout_row)
  dialog_content:add_child(transport_and_sliders_row)
  dialog_content:add_child(off_buttons_popup) --spacer
  dialog_content:add_child(fx_col_buttons_popups)
  dialog_content:add_child(vb:text{ text = "", id = " text"})
  
  --set up tool for first use
  --set global to current track index 


   g_stored_track_index = song.selected_track_index
 
 
  ---------------------------
  --make the selected [Note X] button orange if a note column is selected
  if song.selected_note_column_index ~= 0 then
    --NOTE COL SELECTED--
    vb.views["Note Column Button "..song.selected_note_column_index].color = COLOR_ORANGE 
    --run function to get traget note column (whole pattern_track)
    set_g_slider_lane_target_note_col()
    --trigger notifier to target the first popup of the [NOTE] buttons.
    --i.e. volume_and_note_target_notifier() (found in upper panel functions) 
    g_target_notifier_tab_notes[2]()

    --set parameter target button colors 
    for j = 1,5 do  
      vb.views["note col target "..j].color = COLOR_GREY
    end
    --selected button
    vb.views["note col target "..DEFAULT_NOTE_COL_TARGET_VALUE].color = COLOR_ORANGE
    --set global for selected note column index 
    g_target.column_index = song.selected_note_column_index 
  else --FX COL SELECTED--
    g_target.column_index = song.selected_effect_column_index
    set_g_slider_lane_target_effect_col()
    g_target_notifier_tab_fx[g_target.column_index]()
    fx_column_buttons_notifier(g_target.column_index) --call notifier function to select for this fx column
  end
  
  --store the current track index in the global for flagging track changes
  g_stored_track_index = song.selected_track_index
  --if starting on a note column
  if song.selected_note_column_index ~= 0 then
    g_stored_column.index = song.selected_note_column_index
    g_stored_column.type = "note"
    g_target.column_type = "note"
  else --fx column
    g_stored_column.index = song.selected_effect_column_index
    g_stored_column.type = "effect"
    g_target.column_type = "effect"
  end

  --if there are values already in fx column then set the fx popup targets
  --set timer running to true so that notifiers aren`t called
  
  get_fx_column_targets_for_popups()
  
  ---------------------------------------------------------------------
  --set popup to show off buttons/ x buttons visible configuration -- *****replace with notifier function call*** 
  ---------------------------------------------------------------------
 -- options.show_off_or_x_buttons_popup.value = 1
  if options.show_off_or_x_buttons_popup.value == 1 then
       
     for i = 1, tool_range do
       vb.views["off button"..i].visible = true 
       vb.views["X button"..i].visible = false
      -- vb.views["note name textfield"..i].visible = true
     end
   elseif options.show_off_or_x_buttons_popup.value == 2 then
     for i = 1, tool_range do   
       vb.views["off button"..i].visible = false 
       vb.views["X button"..i].visible = true
    --  vb.views["note name textfield"..i].visible = false
     end
   
   --elseif options.show_off_or_x_buttons_popup.value == 3 then
   --  for i = 1, tool_range do   
    --   vb.views["off button"..i].visible = true 
    --   vb.views["X button"..i].visible = true
   --  end 
   end
   
  
   
   
   --reset timer flag
   timer_running = false
  --------------
  --key Handler
  --------------
  local function my_keyhandler_func(dialog,key)
     --toggle lock focus hack, allows pattern ed to get key input
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
     g_keyhandler_value = key
     return key
  end 
    
  --update globals for first run
  g_original_bpm = song.transport.bpm
  
  -------------------------------------------   
  --close dialog function ON NEW SONG -
  --------------------------------------------
  local function closer(d)
    my_dialog = nil    
    if d and d.visible then
      d:close()
     -- remove_notifiers()
      remove_notifiers()
    end
  end

  ------------------------------------------------------------------------------------------
  ------------------------------------------------------------------------------------------
  --Script dialog
  my_dialog = renoise.app():show_custom_dialog(
      TOOL_NAME, dialog_content,my_keyhandler_func)

  
  --add timer to fire once every 50ms
  if not renoise.tool():has_timer(upper_panel_timer) then
    renoise.tool():add_timer(upper_panel_timer,50)
  end
  
  -- notifier to close dialog on load new song
  renoise.tool().app_release_document_observable:add_notifier(closer,my_dialog)
  -------------------------------------------------------------------------------
  
  --second run of timer
  upper_panel_timer()
end


