--timer to update
--top half of GUI
--
-------------------------------
--LINE 1142 to add lower border
-------------------------------


EMPTY_FX_COLUMN = 15
 
require "_target_notifier_functions"
require "_gui_notifiers"
require "_timer"

---------------------------------------------Sample Commands---------------------------------------------------------
--[[
These commands only apply to samples and will have no effect on the Plugin or MIDI components of an Instrument. When used in the Master track, the effect will be applied to all tracks.

    1 -Axy - Set arpeggio, x/y = first/second note offset in semitones. Using 0 for x or y will use the original note.
    2 -Uxx - Slide pitch up by xx 1/16ths of a semitone (01 is 1/16th of a semitone, 08 is half a semitone, 10 is a whole semitone).
    3 -Dxx - Slide pitch down by xx 1/16ths of a semitone (01 is 1/16th of a semitone, 08 is half a semitone, 10 is a whole semitone).
    4 -Gxx - Glide towards given note by xx 1/16ths of a semitone (01 is 1/16th of a semitone, 08 is half a semitone, 10 is a whole semitone). A value of FF will make the slide instant.
    5 -Ixx - Fade volume in by xx volume units (-I01 inserted 256 times will slide from 0 to full volume, -I7F inserted twice will do the same much faster).
    6 -Oxx - Fade volume out by xx volume units.
    7 -Txy - Set tremolo (regular volume variation), x = speed, y = depth.
    8 -Cxy - Cut volume to x after y ticks (x = volume factor: 0=0%, F=100%).
    9 -Sxx - Trigger sample slice number xx or offset xx.
    10-Bxx - Play sample backwards (xx = 00) or forwards (xx = 01).
    11-Vxy - Set vibrato (regular pitch variation), x = speed, y = depth.
    12-Nxy - Set auto pan (regular pan variation), x = speed, y = depth.
    13-Exx - Set the position of all active Envelope, AHDSR and Fader Modulation devices to offset xx. 
--]]
-----------------------------------------------------------------------------------------------------------------------




--table for beat marker labels in transport buttons
beat_markers = {}
---------------------------------
function calculate_beat_markers()
---------------------------------
  local beat_markers = {}
  local lpb = renoise.song().transport.lpb
  --loop to fill table
  for i = 1,tool_range do
    if (i - 1) % lpb == 0 then 
      beat_markers[i] = "|" --beat marker
    else
      beat_markers[i] = "" --off beat
    end
  end
   return beat_markers
end


-------------------------
--get renoise track names
-------------------------
function get_renoise_track_names()
  local song = renoise.song()
  local track_name_table = {}
  for track = 1,#song.tracks do
    --if song.tracks[track].type == renoise.Track.TRACK_TYPE_SEQUENCER then
      table.insert(track_name_table,song.tracks[track].name)
   -- end
  end
  return track_name_table
end 


table_of_sample_effects = {}
table_of_sample_effects.name_strings = nil
table_of_sample_effects.number_strings = nil

table_of_sample_effects.name_strings = {

 "Arp",--...0AXY", 1
 "Pitch up",--...0UXX", 2
 "Pitch down",--...0DXX", 3
 "Glide",--...0GXX", 4
 "Fade In",--...0IXX",  5
 "Fade Out",--...0OXX", 6
 "Tremolo",--...0TXY", 7
 "Cut",--...0CXY", 8
 "Samp. Offset",--...0SXX", 9
 "Backwards",--...0BXX", 10
 "Vibrato",--...0VXY", 11
 "Auto Pan",--...0NXY",  12
 "Env. offset",--...0EXX", 13
 "DEVICE",--14
 "---" --15 empty
}
table_of_sample_effects.number_strings = {

 "0A",--...0AXY", 1 Arp
 "0U",--...0UXX", 2 Pitch up
 "0D",--...0DXX", 3 Pitch Down
 "0G",--...0GXX", 4 "Glide"
 "0I",--...0IXX", 5 Fade in
 "0O",--...0OXX", 6 Fade out
 "0T",--...0TXY", 7 Tremolo
 "0C",--...0CXY", 8 Cut
 "0S",--...0SXX", 9 Samp Offset
 "0B",--...0BXX", 10 Backwards
 "0V",--...0VXY", 11 Vibrato
 "0N",--...0NXY", 12 Autopan
 "0E",--...0EXX", 13 Env. Offset
 "DEVICE",
 "---"          --14 empty
}
--constant "DEVICE" key in table
USER = 14

-------------------------------------------

-------------------------------------------










--table needed to convert the second character in an fx column to the correct number to display it`s parameter name
--i.e. 3Jxx   Where 3 is the device index and `J` is the parameter-index (== 19)
local param_char_table = {}

param_char_table["1"] = 1
param_char_table["2"] = 2
param_char_table["3"] = 3
param_char_table["4"] = 4
param_char_table["5"] = 5
param_char_table["6"] = 6
param_char_table["7"] = 7
param_char_table["8"] = 8
param_char_table["9"] = 9
param_char_table["A"] = 10
param_char_table["B"] = 11
param_char_table["C"] = 12
param_char_table["D"] = 13
param_char_table["E"] = 14
param_char_table["F"] = 15
param_char_table["G"] = 16
param_char_table["H"] = 17
param_char_table["I"] = 18
param_char_table["J"] = 19
param_char_table["K"] = 20
param_char_table["L"] = 21
param_char_table["M"] = 22
param_char_table["N"] = 23
param_char_table["O"] = 24
param_char_table["P"] = 25
param_char_table["Q"] = 26
param_char_table["R"] = 27
param_char_table["S"] = 28
param_char_table["T"] = 29
param_char_table["U"] = 30
param_char_table["V"] = 31
param_char_table["W"] = 32
param_char_table["X"] = 33
param_char_table["Y"] = 34
param_char_table["Z"] = 35




--[[ SEE NEXT FUNCTION ----
----------------------------------------------------------
function _get_current_target_parameter_for_device_readout()
----------------------------------------------------------
  local song = renoise.song()
 
  --currently selected target i.e "DEVICE"
  local target_text = string.upper(g_target.value_type)
  
  if target_text == "DEVICE" then

    --get characters from the global that holds fx target
    local fx_col_char_one = string.sub(g_user_effect_number_string,1,1)
    local fx_col_char_two = string.sub(g_user_effect_number_string,2,2)
    
    --1) Mixer device uses "P" "L" and "W" as the second character in pattern automation
      --not digits like othe devicres in chain
    if fx_col_char_two == "P" then
      target_text = "Pan"
    elseif fx_col_char_two == "L" then
      target_text = "Volume"
    elseif fx_col_char_two == "W" then
      target_text = "Width"
    else
    
    local dev_idx = tonumber(fx_col_char_one,16)
      --custom table previously declared, is used for conversion chars "1 through Z" to corresponding number "1-35"
      local param_idx = param_char_table[fx_col_char_two] 
      --increment device index (Lua devices count from 1 / pattern commands counts mixer as 0)
      dev_idx = dev_idx + 1 
  
  
      --if no parameter (param == 0) is specified then skip
      if(param_idx ~= nil) and (param_idx > 0)  then 
      
        if dev_idx <= #song.selected_track.devices then
         -- local target_device = song.selected_track:device(dev_idx).display_name
          local target_parameter = song.selected_track:device(dev_idx):parameter(param_idx).name
    
          --update the parameter readout below the popups
         vb.views["param textfield "..g_target.column_index].text = target_parameter
         vb.views["param textfield border"..g_target.column_index].style = "border"
        end
      end
    end
  end
end
--]]
--populates the parameter name boxes (under all pop-ups) when "DEVICE"
--is selected in any of the popups
-------------------------------------------------------
function get_all_target_parameters_for_device_readout()
-------------------------------------------------------
  do return end
 
 
  local song = renoise.song()
 
  --currently selected target i.e "DEVICE"
 -- local target_text = string.upper(g_target.value_type)
  
  --loop through all 8 fx columns 
  for col = 1,8 do
    --set parameter boxes to invisible as default (will be changed back to "border if fx entry is found")
    vb.views["param textfield border"..col].style = "invisible"
    vb.views["param textfield "..col].text = ""
    
    if vb.views["popup col "..col].value == 14 then --14 == "DEVICE" 
      
      --variables to hold the first two characters i.e "12xx" get 1 and 2 (device 1, parameter 2)
      local fx_col_char_one = nil
      local fx_col_char_two = nil
      
      local fx = nil
       
      --loop column-in-pattern to get target device characters
      for line = 1,song.selected_pattern.number_of_lines do
        --get each effect entry in column
        fx = song.selected_pattern_track:line(line):effect_column(col)
        --break loop if entry found
        if fx.number_string ~= "00" then
          break
        end
      end
      
      --get individual characters from the found fx entry
      fx_col_char_one = string.sub(fx.number_string,1,1)
      fx_col_char_two = string.sub(fx.number_string,2,2)
      
      --1) Mixer device uses "P" "L" and "W" as the second character in pattern automation
      --not digits like othe devices in chain
      if fx_col_char_two == "P" then
        target_text = "Pan"
      elseif fx_col_char_two == "L" then
        target_text = "Volume"
      elseif fx_col_char_two == "W" then
        target_text = "Width"
      else
      
      local dev_idx = tonumber(fx_col_char_one,16)
        --custom table previously declared, is used for conversion chars "1 through Z" to corresponding number "1-35"
        local param_idx = param_char_table[fx_col_char_two] 
        --increment device index (Lua devices count from 1 / pattern commands counts mixer as 0)
        dev_idx = dev_idx + 1 
    
        --if no parameter (param == 0) is specified then skip
        if (param_idx ~= nil) and (param_idx > 0) and (param_idx <= #song.selected_track:device(dev_idx).parameters)  then 
        
        
          if dev_idx <= #song.selected_track.devices then
           -- local target_device = song.selected_track:device(dev_idx).display_name
           local target_parameter = song.selected_track:device(dev_idx):parameter(param_idx).name
      
            --update the parameter readout below the popups
            vb.views["param textfield "..col].text = target_parameter
            vb.views["param textfield border"..col].style = "invisible"
          end
        end
      end
    end
  end
end









-------------------------------------------
function get_fx_column_targets_for_popups()
-------------------------------------------
  local song = renoise.song()
  local user_value = false
  --1)--loop all fx columns of selected track to find any present fx data
  --set the popups to the first fx type found there
  for j = 1,8 do
    --if popup shows no effect found it`s default string [---]
    if vb.views["popup col "..j].value == #table_of_sample_effects.name_strings then --works

      --flags
      local found_flag = false
      local user_value = false
      --loop lines in fx column
      for line = 1, #song.selected_pattern_track.lines do
        if found_flag == true then
          break
        end
        local fx = song.selected_pattern_track:line(line):effect_column(j)
        --loop to find match with popup table
        for popup_idx = 1, #table_of_sample_effects.number_strings do
          
          
          --set the popups to correct value
          if fx.number_string == table_of_sample_effects.number_strings[popup_idx] then --get first two characters
            vb.views["popup col "..j].value = popup_idx
            found_flag = true
            break
          end
          --flag that some value has been found, it is potentially a user value
          if fx.number_string ~= "00" then

            user_value = true
            g_user_effect_number_string = fx.number_string
          end
          
          --if we got here (i.e. did not return and a user value was found, then set the popup value here)
          if (user_value == true) and (found_flag == false) then
            vb.views["popup col "..j].value = USER --USER = 14
          end
                
        end --individual popup
      end --lines in pattern
    end--if
  end--for buttons (cols) 1-8
end--function

function show_grab_extra_buttons(val)
   for i = 1,tool_range do
     vb.views["grab extra"..i].visible = val
     --reset text to empty
     vb.views["grab extra"..i].text = ""
     --reset color to grey to empty
     vb.views["grab extra"..i].color = COLOR_GREY
     --reset grab target to nil
     g_grab_extra_target_button = nil
   end
end

--transport text
--options.transport_text_flag.value = "beat markers" --can be "beat markers","pattern line numbers" or "pattern notes" 

------------------------------------
------------------------------------
function main_upper_panel()
------------------------------------
------------------------------------


  --toggle dialog so if its already open then close it.  The timer will catch
  --that it is closed and do the housekeeping like removing notifiers (and timer itself)
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    return
  end
  
  --BUG CHECK
  --in case tool crashed and notifiers weren`t removed
  if remove_notifiers() == true then
    tprint("notifiers removed on start")
  end
  
  --renoise song object
  local song = renoise.song()
  
  --get song bpm
  g_original_bpm = renoise.song().transport.bpm
  --calculate beat markers table --"|" marker spacing in transport buttons
  beat_markers = calculate_beat_markers()
  
  
  --------------------------------------
  local function notifier_show_bottom_panel()
  ------------------------------------
    --toggle view if already constructed
    if vb.views["bottom panel"] then
      if vb.views["bottom panel"].visible == true then
        vb.views["bottom panel"].visible = false
        return
      else
        vb.views["bottom panel"].visible = true
        return
      end
    end
    --else create bottom panel  
     dialog_content:add_child(show_bottom_panel())
   end

  --set globals
  -------------
  --assign global vb to viewbuilder
  vb = renoise.ViewBuilder()
  
  ----------------------------------------------
  --GUI
  ----------------------------------------------     

  

   
  
  --get renoise note names
  ---------------------------------------
  local function get_renoise_note_names()
  ---------------------------------------
    local note_name_table = {}
    local note_strings = {
    "C-", "C#", "D-", "D#", "E-", "F-", 
    "F#", "G-", "G#", "A-", "A#", "B-"
      }
    
    local counter = 1
    for octave = 1,10 do
      for note = 1,#note_strings do
        note_name_table[counter] = note_strings[note]..(octave-1) 
        counter = counter+ 1
      end
    end
    return note_name_table
  end
-------------------------------



  --get renoise note names in reverse order (otherwise higest notes occupy lowest position in the popup)
  ---------------------------------------
  local function get_renoise_note_names_reverse_order()
  ---------------------------------------
    local note_name_table = {}
    local note_strings = {
    "C-", "C#", "D-", "D#", "E-", "F-", 
    "F#", "G-", "G#", "A-", "A#", "B-"
      }
    
    local counter = 1
    --loop backwards octaves
    for octave = 10,1,-1 do
      for note = #note_strings,1,-1 do
        note_name_table[counter] = note_strings[note]..(octave-1) 
        counter = counter + 1
      end
    end

    return note_name_table
  end
  
 -- rprint(get_renoise_note_names())
  --rprint(get_renoise_note_names_reverse_order())
-------------------------------
-----------  
--build GUI
-----------



--main rows of the tool gui
--vb:rows{}
local solo_row = vb:row {margin = 4} --set default margin for each row `4`
local transport_and_sliders_row = vb:row {margin = 4}
local fx_col_buttons_popups = vb:row {margin = 4}
-------------------------------


local active_buttons = nil
local groove_sliders = nil
local note_column_select_buttons = nil

----------------------------------------------------------------------------------  
----------------------------------1)TOP ROW---------------------------------------
--[solo] [1/2 bpm] [track select] [rename track] [default note] [clear silent notes]
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
-----------------------------------
local function chord_mode_enabled()
-----------------------------------  
  if options.chord_mode_enabled.value == true then
    return COLOR_RED
  else
    return COLOR_GREY
  end
end

--build row
local row_one = vb:row{ 
  -- margin = 4,
   
   vb:button{
    text = "Solo",
    width = 80,
    height = 20,
    color = COLOR_GREY,
    id = "solo button",
    notifier = function() solo_button_notifier() end
   },
    
   vb:button{
    text = "1/2 BPM",
    color = COLOR_GREY,
    height = 20,
    width = 65,
    id = "bpm button",
    notifier = function() bpm_button_notifier() end
   },
   
   vb:popup{
    width = 120,
    height = 20,
    items = get_renoise_track_names(),
    value = song.selected_track_index,
    id = "track names",
    notifier = function(value) 
      --return early if it was the timer changing the value and not the user
      if timer_running == true then
        return 
      end
      --run notifier function
      track_names_popup_notifier(value)
      end
   },
                
   vb:textfield{
    id = "rename track txt",
    notifier = function(value) rename_track_textfield_notifier(value) end 
   }, 
   
   vb:text{
    text = ""
   },  --spacer
   
  vb:button{
    id = "R button",
    color = COLOR_GREY,
    text = "C-4",--reset target note popup to C-4
    notifier = function()
      
      local popup = vb.views["target note"]
      --if value not C-4 then set it to that
      if popup.value ~= 72 then
        popup.value = 72--49 --C-4
        return
      end
      --[[
      --if value is already C-4 toggle to the first note value in the sequence
         
      local text = nil
      --loop note textfields
      for i = 1,tool_range do
        text = vb.views["note name textfield"..i].value
        if text ~= "" and text ~= "OFF" then
          --legitimate note found
          break
        end
        if i == tool_range then --no note found so return
          status("No Note Found")
          return
        end
      end
      --loop target not popup to find index that corresponds to note and set it
      local items = #vb.views["target note"].items
      for i = 1,items do
        if text == vb.views["target note"].items[i] then
           vb.views["target note"].value = i
           break
         end
       end--]]
        
    end
   },
  
  vb:button{ --Button sets the target note popup to the first found note in the lane
    id = "1st button",
    color = COLOR_GREY,
    id = "1st button",
    text = "1st",--reset target note popup to C-4
    notifier = function()
      --note-text holder
      local text = nil
      --loop note textfields
      for i = 1,tool_range do
        text = vb.views["note name textfield"..i].value
        if text ~= "" and text ~= "OFF" then
          --legitimate note found
          break
        end
        if i == tool_range then --no note found so return
          status("No Note Found")
          return
        end
      end
      --loop target not popup to find index that corresponds to note and set it
      local items = #vb.views["target note"].items
      for i = 1,items do
        if text == vb.views["target note"].items[i] then
           vb.views["target note"].value = i
           break
         end
       end
    end
   },
 
  
     
   vb:popup{
    id = "target note",
    items = get_renoise_note_names_reverse_order(), --get_renoise_note_names(),
    value = (120 - g_target.default_note ), --default note (saved as global)
    notifier = function(value)
      --update global, (used forsetting initial value of popup [per-session, not an options.value],
      --value has to be (120 -) as popup order is reversed from default
      g_target.default_note = 120 - value
     --if timer change popup value, don`t run 
     if timer_running == true then 
       return
     end
     set_default_note_for_entry_popup(value)
     --run timer to update gui quicker in [G] mode
     upper_panel_timer() --loop stopped by flag above
   end
  },
   
  
   
   vb:button{
    id = "grab button",
    color = COLOR_GREY,
    width = 50,
    text = "G",
    notifier = function()
      if vb.views["grab button"].color[1] == COLOR_GREY_FLAG then
        
        --show grab extra buttons, checking checkbox enabling them first
        if vb.views["grab lane enabled cb"].value == true then
          show_grab_extra_buttons(true)
        end
        vb.views["grab button"].color = GRAB_BUTTON_ENABLED_COLOR
        --disable song edit mode
        song.transport.edit_mode = false
        --enable note + velocity target
       
       -- volume_and_note_target_notifier
        note_column_target_notifier(2)  --(value,button)
    
       -- g_target.value_type needs to  == "note + volume"
       g_target_notifier_tab_notes[2]()
       
              
      else
        --hide grab extra buttons
        show_grab_extra_buttons(false)
        --turn button grey, timer takes care of the rest
        vb.views["grab button"].color = COLOR_GREY
      end
    end
     
     --  set_default_note_for_entry_popup(value) end
   },
   
   vb:checkbox{
    id = "grab lane enabled cb", 
    value = options.grab_target_lane_checkbox_enabled.value, --stored in options
    notifier = function(value)
      if timer_running == true then
        return
      end
      --update prefernce to store state
      options.grab_target_lane_checkbox_enabled.value = value
      --show lane if we are in grab mode
      if value == true then
        if vb.views["grab button"].color[1] == GRAB_BUTTON_ENABLED_COLOR_FLAG then
          show_grab_extra_buttons(true)
        end
      else --hide lane
        show_grab_extra_buttons(false)
      end
    end
    },
   
   vb:text{}, --spacer
                
   vb:button{
     text = "Clear Silent Notes",
     id = "Clear Silent Notes",
     color = COLOR_GREY,
     notifier = function() clear_silent_notes_button() end,
    }, 
    
    vb:text{width = 30}, --spacer
   vb:button{
     text = "CHORD MODE",
     id = "chord mode",
     color = chord_mode_enabled(),
     notifier = function()
                  --When button red it is caught/flagged in the function 'main_sliders_notifier(value,button)' 
                  --in file '_gui_notifiers.lua'
                  
                  --toggle button color 
                  if vb.views["chord mode"].color[1] == COLOR_GREY_FLAG then
                    vb.views["chord mode"].color = COLOR_RED
                    --update preference
                    options.chord_mode_enabled.value = true 
                  else
                    vb.views["chord mode"].color = COLOR_GREY
                    --update preference
                    options.chord_mode_enabled.value = false
                  end
               end,
     
    },
    
    
          
   } --end of first row               
                      
solo_row:add_child(row_one) 
----------------------------
--end of first row

       
-------------------2)-SELECT NOTE COLUMN BUTTONS AND POPUPS----------------
--[NOTE 1] [NOTE 2] [NOTE 3]....etc
--[Volume] [pitch] [Empty]....etc
------------------------------------------------------------------------     
--view to be added into dialog_content later
local note_col_buttons_and_popups  = vb:row{
                          margin = 4
                     } 
--[[      
--table of targets to populate the popups
local table_of_note_effects = {
 "Note + Vol.",
 "Volume",
 "Pitch",
 "Delay",
 "Pan",
}
--]]

local table_of_note_effects = {
 "Pitch",
 "Note + Vol.",
 "Volume",
 "Pan",
 "Delay",
}

--loop to create for all note columns
-------------------------------------
for i = 1, 12 do --number of available fx columns in renoise
  local note_col_popup = vb:row{ 
    vb:column{
     --BUTTONS TO CHOOSE WHICH NOTE COLUMN TO TARGET
     -----------------------------------------------
     vb:button{
      text = "NOTE "..i,
      id = "Note Column Button "..i,
      height = 26,
      width = 90,
      color = COLOR_GREY,
      notifier = function()
        --return early if it was the timer changing the value and not the user
        if timer_running == true then
          return 
        end
        --run notifier function
        note_selection_buttons_notifier(i)
        --now run timer() to speed up update of sliders 
        --(better than doing inside notfier function in case it is called elsewhere and we don`t want timer fired)
        upper_panel_timer()    
        end
     },
     --POPUPS TO CHOOSE WHICH NOTE COLUMN VALE TO TARGET "VOL""PITCH"....                    <<< ---DELETE THESE
      -----------------------------------------------------------------
    --[[ vb:popup{ 
      items = table_of_note_effects,
      id = "Note popup col "..i,
      value = 1,--defalt to Vol
      width = 90,
      notifier = function(value) note_popup_selection_notifier(value,i) end,
      visible = false,
      }, --]] 
     }
    }
 --add into view note_col_buttons_and_popups
 note_col_buttons_and_popups:add_child(note_col_popup)
end --for loop






--END OF NOTE COLUMN  BUTTONS  


local note_col_target_buttons = vb:horizontal_aligner{vb:text{text = ""}} --,mode = "center"}

--loop to add buttons
for i = 1,5 do

  local button = vb:button{
    id = "note col target "..i,
    text = table_of_note_effects[i],
    width = 80,
    color = COLOR_GREY,
    notifier = function()
        --return early if it was the timer changing the value and not the user
        if timer_running == true then
          return 
        end
        --run notifier function
        note_column_target_notifier(i)
      end
   }
  note_col_target_buttons:add_child(button)
end



--checkbox to show rotaries
-----------------------
local delay_rotaries =
  vb:row{ 
    vb:checkbox{
      id = "del rotaries cbox",
      value = options.delay_rotaries_visible.value,
      notifier = function(value)
                  --update preference
                  options.delay_rotaries_visible.value = value
                  --show or hide delay rotaries
                  for i = 1, tool_range do
                    vb.views["delay val rotary vis"..i].visible = value
                  end
                end,
      },
    vb:text{
     text = "D. Lane",
    }
    }
--------------------  
  --add to target buttons row
-----------------------------
note_col_target_buttons:add_child(delay_rotaries)


--row 3) updates the user on what the target is:
---------------------------------(3)-----------------------------------------------  
local target_readout_row =
  vb:horizontal_aligner{
   margin = 4,
   mode = "center",
  
    vb:row{
     style = "border",
      vb:text{
      id = "Selected target",
      font = "big",
      height = 25,
      width = 300,
      },
     },
     --[[
   vb:rotary{ --real time record values
     height = 25,
     notifier = function(value)
                  upper_panel_timer()
                  local song = renoise.song()
                  --song selected_line_index
                  local selected_line = song.selected_line_index
                  --caluculate the offset to fit the tools range
                  local target_pat_line = (selected_line - (g_tool_start_line - 1))

                  --calculate the correct renoise pattern line index with global g_tool_start_line
                 -- local target_pat_line = (song.selected_line_index) + g_tool_start_line -1)

                  value = value * (vb.views["slider"..target_pat_line].max/1)
                  
                  
                  vb.views["slider"..target_pat_line].value = value
                  
                
                end
   
   }--]]
 }


---------------------------------(4)-----------------------------------------------    

----------------------------------------------------------------------------------
--[Transport Buttons] [MAIN SLIDERS] [Note Textfields][OFF Buttons](red when note off)
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------   
------------------------------------
local function get_button_color(num)
------------------------------------
  if num < 5 then
    return COLOR_GREEN
  else
    return COLOR_GREY
  end
end
---------------------------------------------------------------------------------
--row 4) TRANSPORT BUTTONS + SLIDERS  [Note Textfields] [OFF Buttons](red when note off)
---------------------------------------------------------------------------------

--loop to create buttons 
for button = 1,tool_range do
  
  local panel = 
    vb:column{
      
     vb:button{ ------------transport buttons-----
      id = "transport button"..button, 
      width = SLIDER_WIDTH,
      height = 20,
      text = beat_markers[button],--can be changed for the function?
      color = get_button_color(button),
      notifier = function() transport_buttons_notifier(button) end          
     },
     
 

      vb:button{ ------------grab buttons-----
       id = "grab extra"..button, 
       width = SLIDER_WIDTH,
      height = 20,
      -- text = beat_markers[button],--can be changed for the function?
      color = COLOR_GREY,
      visible = false,
      notifier = function()
      
       --toggle this button to grey and inactive
       if vb.views["grab extra"..button].color[1] == COLOR_RED_FLAG then
         vb.views["grab extra"..button].color = COLOR_GREY
         vb.views["grab extra"..button].text = ""
         g_grab_extra_target_button = nil
         return
       end 
       
       --or enable
       if vb.views["grab button"].color[1] == GRAB_BUTTON_ENABLED_COLOR_FLAG then
         --turn buttons grey and clear text
        for i = 1,tool_range do
          vb.views["grab extra"..i].color = COLOR_GREY
          vb.views["grab extra"..i].text = ""
        end
        --get target note text
        local text = vb.views["target note"].items[vb.views["target note"].value]
        --turn this button red
        vb.views["grab extra"..button].color = COLOR_RED
        vb.views["grab extra"..button].text = text
        --update global
        g_grab_extra_target_button = button

        return
      end 
      end          
     },
     
 
     ------------------------------------------------------------------------------------------------------------------------       
     ----------------------------------------------*********  SLIDERS  **********--------------------------------------------
     ------------------------------------------------------------------------------------------------------------------------
     
     
     
     
     vb:column{
       id = "slider column "..button,--enables hilighting sliders on the beat
       
       vb:slider{ 
        width = SLIDER_WIDTH,
        height = 150,
        min = 0,
        max = VEL_SLIDER_MAX,
        id = "slider"..button,
        steps = {1,10},
        
        notifier = function(value)
                     if timer_running then
                      return
                     end
                     --print(value)
                    
                     main_sliders_notifier(value,button) 
                   end 
       },
     },
     
      
      ---EXTRA DELAY ROTARIES---
    vb:row{
     margin =2,
     id = "delay val rotary vis"..button,
     vb:rotary{ 
      width = SLIDER_WIDTH-4,
      height = SLIDER_WIDTH-4,
      id = "delay val rotary"..button,
      max = 255,
      visible = true,
       notifier = function(value)
                    --return if the timer is trunning
                    if timer_running then
                      return
                    end
                    
                    local song = renoise.song()
                    --make sure delay column is visible
                    song.selected_track.delay_column_visible = true 
                    --calculate the correct renoise pattern line index with global g_tool_start_line
                    local target_pat_line = (button + g_tool_start_line -1)   
                    local col = g_target.column_index 
                    --add value to the delay column
                    if g_target.column_type == "note" then
                     g_target.all_note_columns[col][target_pat_line].delay_value = value
                    end
                    
                    
                    --If CHORD MODE enabled then update all other columns
                    -----------------------------------------------------
                    local trk = song.selected_track
                    local columns = song.selected_track.visible_note_columns
                    local pat_trk = song.selected_pattern_track
                    
                    
                    if vb.views["chord mode"].color[1] ~= COLOR_GREY_FLAG then 
                      --get current line
                      local cur_line = pat_trk:line(target_pat_line)
                      --loop columns
                      for col = 1,columns do
                        --check if note present
                        if cur_line:note_column(col).note_value < 121 then
                          cur_line:note_column(col).delay_value = value --cur_line:note_column(col).delay_value
                        end
                      end 
                    end
                  end  
     },
    },
 
    

     
     
         ---NOTE TEXTFIELDS---
     vb:textfield{ 
      text ="---",-- get_renoise_track_names()[49],
      width = SLIDER_WIDTH,
      id = "note name textfield"..button,
      --visible = false,
      notifier = function(text)
        if timer_running then
          return
        end
        note_name_textfield_notifier(text,button) end  
     },
     
     
     
  
  
     vb:button{
      id = "X button"..button,
      width = SLIDER_WIDTH,
     -- text = "x",
     -- height = 36,
      color = COLOR_GREY,
      visible = true,
      notifier = function() 
          --g_target = {}
          --g_target.column_index = 1
          --g_target.column_type = "note"
          --g_target.value_type = "volume"
          --g_target.all_note_columns = nil
          --g_target.all_fx_columns = nil
          
          --rprint(g_target)
          --print(g_target.column_index)
         
          local song = renoise.song()       
          if g_target.column_type == "note" then
  
            local note = song.selected_pattern_track:line(button + tool_offset):note_column(g_target.column_index)
            if (g_target.value_type == "pitch") and note.note_string ~= "OFF" then
              if vb.views["X button"..button].color[1] == COLOR_RED_MILD_FLAG then
                note.note_string = "---"
                note.instrument_string = ".."
              else
                note.note_string = "C-4"
              end
            elseif g_target.value_type == "note + volume" and note.note_string ~= "OFF" then
              --delete if button red (volume will be 00 in this case)
              if vb.views["X button"..button].color[1] == COLOR_RED_MILD_FLAG then
                note.note_string = "---"
                note.volume_string = ".."
                note.instrument_string = ".."
              else
                note.volume_string = "00"
              end
             -- note.instrument_string = ".."
            elseif g_target.value_type == "volume" then
              note.volume_string = ".."
            elseif g_target.value_type == "pan" then
              note.panning_string = ".."
            elseif g_target.value_type == "delay" then
              note.delay_string = ".."
            end
          elseif g_target.column_type == "effect" then
            local fx = song.selected_pattern_track:line(button + tool_offset):effect_column(g_target.column_index)
            fx:clear()
         end
         upper_panel_timer()
       end
       },

              
     vb:button{
      id = "off button"..button, --Blue when OFF / Clear Note buttons
      width = SLIDER_WIDTH,
     -- text = "o",
      color = COLOR_GREY,
      notifier = function() off_buttons_notifier(button) end

     },
    
     
     
    }
  --highlight sliders on the beat  
  --if (button - 1) % 4 == 0 then
  -- vb.views["slider column "..button].style = "panel"
   --vb.views["beat marker text"..button].text ="    >"
 -- end

  --add each iteration into transport_and_sliders_row
  transport_and_sliders_row:add_child(panel) 
end --end loop

-----------------------------------------------------
--end of row 4---------------------------------------

--create a long text spacer
---------------------------
local spacer_text_1 = " "
for i = 1,322 do
  spacer_text_1 = spacer_text_1.." "
end
--------------------------
local off_buttons_popup =
 vb:row{
   vb:text{
    text = spacer_text_1
   },
   vb:switch{
    id = "off vs x popup",
    items = {"OFF ","X "},--,"Both"},
    value = options.show_off_or_x_buttons_popup.value,-- options.show_off_or_x_buttons_popup.value,
    width = 150,
    notifier = function(value)
     --update preference
     options.show_off_or_x_buttons_popup.value = value
     if vb.views["off vs x popup"].value == 1 then
       
       for i = 1, tool_range do
         vb.views["off button"..i].visible = true 
         vb.views["X button"..i].visible = false
         
        -- vb.views["note name textfield"..i].visible = true
       end
     elseif vb.views["off vs x popup"].value == 2 then
       for i = 1, tool_range do   
         vb.views["off button"..i].visible = false 
         vb.views["X button"..i].visible = true
       -- vb.views["note name textfield"..i].visible = false
       end
      --[[
     elseif vb.views["off vs x popup"].value == 3 then
       for i = 1, tool_range do   
         vb.views["off button"..i].visible = true 
         vb.views["X button"..i].visible = true
       end --]]  
     end
   
   
   
   
   end
  }
 }




--row 5)-----------------------------------------------------------------------       
--------------------SELECT COLUMN BUTTONS AND POPUPS---------------------------
--[FX 1]  [FX 2]  [FX 3]....etc
--[Glide] [pitch] [Empty]....etc
-------------------------------------------------------------------------------      
      

for i = 1, song.selected_track.max_effect_columns do --number of available fx columns in renoise
  
local fx_col_popups = vb:row{  --change name here-

    vb:column{
      vb:button{
       text = "FX "..i,
       id = "Fx Column Button "..i,
       height = 26,
       width = 90,
       color = COLOR_GREY,
       notifier = function()
                   if timer_running == true then 
                      return
                   end 
                   fx_column_buttons_notifier(i)
                   --refresh popup target automatically on second press
                  
                   --reset the button to "---"
                   vb.views["popup col "..i].value = #table_of_sample_effects.name_strings 
                   --now run get_fx fn to capture target from pattern data (if is an empty col then it just remains "---")
                   get_fx_column_targets_for_popups()
                   --populate the parameter tables below the popup targets when "DEVICE" is selected
                   get_all_target_parameters_for_device_readout()
                  
                   
                   --run timer to update gui quicker/ smoothly
                   upper_panel_timer()  
                   end        
      },

     vb:popup{
      items = table_of_sample_effects.name_strings,
      id = "popup col "..i,
      value = EMPTY_FX_COLUMN,--defalt to empty "---"
      width = 90,
      notifier = function(value) 
                   
                 
      
                   if timer_running == true then
                     return
                   end
                   --use same notifier as button above (both get set at the same time)
                   fx_column_buttons_notifier(i)
                  -- fx_column_popups_notifier(value,i)
                  --populate the parameter tables below the popup targets when "DEVICE" is selected
                   get_all_target_parameters_for_device_readout()
                   
                 end
     },
     --[[ 
     ---parameter readouts/ lables for when DEVICE is selected
     vb:row{
      id = "param textfield border"..i, 
      style = "invisible",--start off invisible and only show when "DEVICE" is chosen
      
       vb:text{
       id = "param textfield "..i,

      height = 20,
      width = 90,
      }
     } --]] 
    } 
   }
 
  fx_col_buttons_popups:add_child(fx_col_popups) 
end--end row 5)


------------------------------------
------------------------------------  
local refresh_button = vb:button{
       text = "Delete Junk",
       notifier = function()
                    local song = renoise.song()
                    --loop all fx cols
                    for col = 1,8 do
                      local first_fx_num = song.selected_pattern_track:line(1):effect_column(col).number_value
                     
                     for line = 1,#song.selected_pattern_track.lines do
                       --get first fx entry
                       if first_fx_num == nil then
                         if song.selected_pattern_track:line(line):effect_column(col).number_value ~= 0 then
                           first_fx_num = song.selected_pattern_track:line(line):effect_column(col).number_value
                         end
                       end
                       --compare subsequent entries and delete any that don`t match
                        if song.selected_pattern_track:line(line):effect_column(col).number_value ~= first_fx_num then
                          --delete fx as out of place
                          song.selected_pattern_track:line(line):effect_column(col).number_string = "00"
                          song.selected_pattern_track:line(line):effect_column(col).amount_string = "00"
                        end
                      end--lines
                   
                    end--cols
                   end,--function
  
             }
             
             --[[
---------------------------------------------------
local select_all_button = vb:button{
        text = "Select Active",
        notifier = function()
                    
                    local song = renoise.song()
                    local first_fx_line = nil
                    local last_fx_line = nil
                    local col = song.selected_effect_column_index
                    
                    
                    
                    --loop to get first and last entries
                    for line = 1,#song.selected_pattern_track.lines do
                      --get position of first value
                      if (first_fx_line == nil) and (song.selected_pattern_track:line(line):effect_column(col).is_empty == false) then
                        first_fx_line = line--song.selected_pattern_track:line(line):effect_column(col).number_value
                      end
                      --get position of last value
                      if (song.selected_pattern_track:line(line):effect_column(col).is_empty == false) then
                        last_fx_line = line--song.selected_pattern_track:line(line):effect_column(col).number_value
                      end
                    end
                    
                    --return and show status if no range is found
                    if (first_fx_line == nil) or (last_fx_line == nil) then
                      status("No Range Found")
                      return
                    end
                    
                    --selection `sees` columns as the same so we need to add the visible note columns to offset
                    local visible_note_cols = song.selected_track.visible_note_columns
                    col = col + visible_note_cols 
                    
                     --create a selection table
                    local select = {
                      start_line = first_fx_line,
                      start_track = song.selected_track_index,
                      start_column = col,
                      end_line = last_fx_line,
                      end_track = song.selected_track_index,
                      end_column = col ,}
                    
                    --update renoise selection
                    song.selection_in_pattern = select

                   end
        


             }--]]


fx_col_buttons_popups:add_child(refresh_button)
--fx_col_buttons_popups:add_child(select_all_button)   
 
   --ASSEMBLE GUI
  --------------------------------------------------------
  --------------------------------------------------------
  --dialog content: master cvb:column{}
  --will contain all of gui; passed to renoise.app():show_custom_dialog()
  dialog_content = vb:column{ margin = 6 }
  --add views
  dialog_content:add_child(solo_row)
  dialog_content:add_child(note_col_buttons_and_popups)
  dialog_content:add_child(note_col_target_buttons)
  dialog_content:add_child(target_readout_row)
  dialog_content:add_child(transport_and_sliders_row)
  dialog_content:add_child(off_buttons_popup) --spacer
  dialog_content:add_child(fx_col_buttons_popups)
 
 -- dialog_content:add_child(vb:text{ text = "", id = " text"})
  
  --set up tool for first use
  --set global to current track index 
   g_stored_track_index = song.selected_track_index
   --set col taget index to 1
   g_slider_lane_two_col_target_index = 1
 
 
  ---------------------------
  --make the selected [Note X] button orange if a note column is selected
  if song.selected_note_column_index ~= 0 then
    --NOTE COL SELECTED--
    vb.views["Note Column Button "..song.selected_note_column_index].color = COLOR_ORANGE 
    --run function to get traget note column (whole pattern_track)
    set_g_slider_lane_target_note_col()
    --trigger notifier to target the first popup of the [NOTE] buttons.
    --i.e. volume_and_note_target_notifier() (found in upper panel functions) 
    g_target_notifier_tab_notes[2]()

    --set parameter target button colors 
    for j = 1,5 do  
      vb.views["note col target "..j].color = COLOR_GREY
    end
    --selected button
    vb.views["note col target "..DEFAULT_NOTE_COL_TARGET_VALUE].color = COLOR_ORANGE
    --set global for selected note column index 
    g_target.column_index = song.selected_note_column_index 
  else --FX COL SELECTED--
    g_target.column_index = song.selected_effect_column_index
    set_g_slider_lane_target_effect_col()
    g_target_notifier_tab_fx[g_target.column_index]()
    fx_column_buttons_notifier(g_target.column_index) --call notifier function to select for this fx column
  end
  
  --store the current track index in the global for flagging track changes
  g_stored_track_index = song.selected_track_index
  --if starting on a note column
  if song.selected_note_column_index ~= 0 then
    g_stored_column.index = song.selected_note_column_index
    g_stored_column.type = "note"
    g_target.column_type = "note"
  else --fx column
    g_stored_column.index = song.selected_effect_column_index
    g_stored_column.type = "effect"
    g_target.column_type = "effect"
  end

  --if there are values already in fx column then set the fx popup targets
  --set timer running to true so that notifiers aren`t called
  
  get_fx_column_targets_for_popups()
  --populate the parameter tables below the popup targets when "DEVICE" is selected
  get_all_target_parameters_for_device_readout()
  
  ---------------------------------------------------------------------
  --set popup to show off buttons/ x buttons visible configuration -- *****replace with notifier function call*** 
  ---------------------------------------------------------------------
 -- options.show_off_or_x_buttons_popup.value = 1
  if options.show_off_or_x_buttons_popup.value == 1 then
       
     for i = 1, tool_range do
       vb.views["off button"..i].visible = true 
       vb.views["X button"..i].visible = false
      -- vb.views["note name textfield"..i].visible = true
     end
   elseif options.show_off_or_x_buttons_popup.value == 2 then
     for i = 1, tool_range do   
       vb.views["off button"..i].visible = false 
       vb.views["X button"..i].visible = true
    --  vb.views["note name textfield"..i].visible = false
     end
   
   --elseif options.show_off_or_x_buttons_popup.value == 3 then
   --  for i = 1, tool_range do   
    --   vb.views["off button"..i].visible = true 
    --   vb.views["X button"..i].visible = true
   --  end 
   end
   
 
    
   
   
   
   ----------------------------------------------------------------
   --reset timer flag
   timer_running = false
  --------------
  --key Handler
  --------------
  local function my_keyhandler_func(dialog,key)
     --toggle lock focus hack, allows pattern ed to get key input
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
     g_keyhandler_value = key
     return key
  end 
    
  --update globals for first run
  g_original_bpm = song.transport.bpm
  
  -------------------------------------------   
  --close dialog function ON NEW SONG -
  --------------------------------------------
  local function closer(d)
    my_dialog = nil    
    if d and d.visible then
      d:close()
     -- remove_notifiers()
      remove_notifiers()
    end
  end

  ------------------------------------------------------------------------------------------
  ------------------------------------------------------------------------------------------
  --Script dialog
  my_dialog = renoise.app():show_custom_dialog(
      TOOL_NAME, dialog_content,my_keyhandler_func)

  
  --add timer to fire once every 50ms
  if not renoise.tool():has_timer(upper_panel_timer) then
    renoise.tool():add_timer(upper_panel_timer,50)
  end
  
  -- notifier to close dialog on load new song
  renoise.tool().app_release_document_observable:add_notifier(closer,my_dialog)
  -------------------------------------------------------------------------------
  
  --second run of timer
  upper_panel_timer()
  
  --on tool startup, set the selected note to the first note found
  ---------------------------------------
  local function set_note_to_first_found()
  ----------------------------------------
    --note-text holder
    local text = nil
    --loop note textfields
    for i = 1,tool_range do
      text = vb.views["note name textfield"..i].value
      if text ~= "" and text ~= "OFF" then
        --legitimate note found
        break
      end
      if i == tool_range then --no note found so return
        status("No Note Found")
        return
      end
    end
    --loop target note popup to find index that corresponds to note and set it
    local items = #vb.views["target note"].items
    for i = 1,items do
      if text == vb.views["target note"].items[i] then
         vb.views["target note"].value = i
         break
       end
     end
   end
   --run local fn
   set_note_to_first_found()


end


