--------------------------------
--NOTIFIERS FOR THE GUI ELEMENTS
--------------------------------

--------------------------------
function solo_button_notifier()
--------------------------------
  local song = renoise.song()
  if vb.views["solo button"].color[1] == COLOR_GREY_FLAG then
    vb.views["solo button"].color = COLOR_YELLOW
    --extra solo needed if track flagged as solo already
    if song.selected_track.solo_state == true then
      song.selected_track:solo()
    end  
    song.selected_track:solo()
  else
  vb.views["solo button"].color = COLOR_GREY
    for i = 1,#song.tracks do
      song:track(i).mute_state = renoise.Track.MUTE_STATE_ACTIVE
    end
  end
end

------------------------------
function bpm_button_notifier()
------------------------------
  local song = renoise.song()
  --grey so halve bpm
  if vb.views["bpm button"].color[1] == COLOR_GREY_FLAG then
    g_original_bpm = song.transport.bpm
    song.transport.bpm = 0.5 * song.transport.bpm --NEEDS BOUNDS CHECKING 
    vb.views["bpm button"].color = COLOR_YELLOW
  else--orange so double bpm to original
    song.transport.bpm = g_original_bpm
    vb.views["bpm button"].color = COLOR_GREY
  end
end

------------------------------------------         
function track_names_popup_notifier(value)
------------------------------------------
  --precaution
  if (timer_running == true) then
     return
  end
  --change track_selection to match what the user chooses in the popup
  renoise.song().selected_track_index = value
  --update items in case something changed
  vb.views["track names"].items = get_renoise_track_names()
end

-----------------------------------------------
function rename_track_textfield_notifier(value)
-----------------------------------------------
  --Stops track renaming again when field is cleared below
  if value == "" then
    return
  end
  --timer flag set
  bypass_timer = true
  local song = renoise.song()
  --rename track in renoise
  song.selected_track.name = value
  --update popup list
  vb.views["track names"].items = get_renoise_track_names()
  --clear current textfield when done
  vb.views["rename track txt"].value = ""
  --timer flag reset
  bypass_timer = false
end

--------------------------------------------------
function set_default_note_for_entry_popup(value) --values are 1-120 
--------------------------------------------------
  --value can be 1 to 120 (10 * octaves of 12 semitone notes)

 
  --popup has reversed note startting at B-9 (value 120) and decending to C-0 (value 1)
  --calculate the correct reversed value here
  --values range from 1-120 `B-9` == 1 C-0 == 120
  --(normally you would subtract from 121 (max + 1) but g_target.default_note counts from zero)
  value = 120 - value
  
  --update global, (used forsetting initial value of popup [per-session, not an options.value],
  g_target.default_note = value
  
 --when timer is not running it means this has been called by user, so update g_keyhandler_value.note as a false keystroke
 --to be picked up by grab function in timer 
 if (timer_running == false) and (vb.views["grab button"].color[1] == GRAB_BUTTON_ENABLED_COLOR_FLAG) then
   
   --need to offset value to count from 1 again (not zero)
   value = value + 1
 
   --do reverse calc that is done in timer to get g_keyhandler_value.note 
   --(messy but works as the conversion is done to value and then back again in timer [translated])
   local octave = renoise.song().transport.octave
   local num = value - (octave*12)
   --send as though key has been pressed for timer to pick up
   g_keyhandler_value = {}
   g_keyhandler_value.note = num - 1

  end
end

------------------------------------
function clear_silent_notes_button() 
------------------------------------
  local song = renoise.song()
  local pattern = song.selected_pattern
  local total_lines = pattern.number_of_lines
  --loop tracks
 -- for track = 1, #song.tracks do
  local track = song.selected_track_index
    if renoise.song():track(track).type == renoise.Track.TRACK_TYPE_SEQUENCER then
      local max_cols = renoise.song():track(track).visible_note_columns
      for line = 1, total_lines do
        --loop visible columns
        for col = 1,max_cols do
          local note_column = pattern:track(track):line(line):note_column(col)
          if (note_column.volume_value == 0) and (note_column.note_string ~= "---") then
            --clear note col
            note_column:clear()
          end
        end
      end
    end
 -- end
end

--notifier for [NOTE x] buttons which select the note column target for the slider lane
--id = "Note Column Button "..i,
--i is the buttons index from loop it is created in
-------------------------------------------- 
function note_selection_buttons_notifier(i)
--------------------------------------------   
  --song
  local song = renoise.song()
  
  --target -will hold index for (Pitch, Note + Vol, Volume etc.) button
  local target = nil
  
  --second click of `this` button defaults to note+velocity
  if (vb.views["Note Column Button "..i].color[1] == COLOR_ORANGE_FLAG) then
   -- song.selected_note_column_index == i then 
    target = 2
    
    --choose OFF lane(buttons under sliders)
   -- vb.views["off vs x popup"].value = 1 

    
  end
  
  --make sure we have a legitimate note col target (Pitch, Note + Vol, Volume etc.)
  --------------------------------------------------------------------------------
  if target ~= 2 or target == nil then
    for j = 1,5 do --loop to find current target
      if vb.views["note col target "..j].color[1] == COLOR_ORANGE_FLAG then
        target = j
        break
      end
    end
    --if no target is found then default to 2
    if target == nil then
      target = 2 --default to Note + Volume
    end
  end
  
  --update global target to match this button
  g_target_notifier_tab_notes[target]()
   
  --turn all target buttons grey 
  for j = 1,5 do  
    vb.views["note col target "..j].color = COLOR_GREY
  end
  --turn target button orange to show selection
  vb.views["note col target "..target].color = COLOR_ORANGE
  -------------------------------------------------------------

  --set global to target the correct note column
  g_target.column_index = i
         
  --make column visible if it is not already
  if song.selected_track.visible_note_columns < i then
    song.selected_track.visible_note_columns = i
  end
  --select column in renoise
  song.selected_note_column_index = i
  --refocus pattern editor
  renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
  renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
  
  --create a selection table
  local select = {
    start_line = 1,
    start_track = song.selected_track_index,
    start_column = song.selected_note_column_index,
    end_line = song.selected_pattern.number_of_lines,
    end_track = song.selected_track_index,
    end_column =song.selected_note_column_index ,}
  
  --update renoise selection
  song.selection_in_pattern = select
  
  --loop to change fx column select buttons to grey as they will now be inactive
  for j = 1,8 do
    vb.views["Fx Column Button "..j].color = COLOR_GREY
  end
  --loop to change all note col buttons to grey
  for j = 1,12 do
    vb.views["Note Column Button "..j].color = COLOR_GREY
  end
  --update this buttons color to orange as it is now selected
  vb.views["Note Column Button "..i].color = COLOR_ORANGE
end

---------------------------------------
function note_column_target_notifier(button_index)--button index --1 = Pitch, 2 = Note + Vol 3) = Volume etc..
--------------------------------------
 
  --loop to see if/which note column is selected in tool
  local tool_selected_note_column = 1 --default to 1
  for i= 1,12 do
    if vb.views["Note Column Button "..i].color[1] ~= COLOR_GREY_FLAG then
      tool_selected_note_column = i
      break
    end
  end
  --run not selection notifier for selected button or default of 1
  note_selection_buttons_notifier(tool_selected_note_column) 
  --update global target to match this button
  g_target_notifier_tab_notes[button_index]()
   
  --turn all target buttons grey (includung this)
  for j = 1,5 do  
    vb.views["note col target "..j].color = COLOR_GREY
  end
  --turn this button orange to show selection
  vb.views["note col target "..button_index].color = COLOR_ORANGE
end

-------------------------------------------
function transport_buttons_notifier(button)
------------------------------------------- 
  local song = renoise.song()
  

             --toggle beat markers and pattern notes
  if options.transport_text_flag.value  == "beat markers" then
    options.transport_text_flag.value = "pattern notes"
  else
  --updat global beat_markers
    beat_markers = calculate_beat_markers()
    for i = 1,tool_range do
      vb.views["transport button"..i].text = beat_markers[i] 
    end
    options.transport_text_flag.value = "beat markers"
  end
  
   --[[
  
  
  
  
  

   -- if g_keyhandler_value ~= nil and g_keyhandler_value.name == "lcontrol" then

    ---------------------------------------------------
    --local function sets edit pos (no bounds checking)
    ---------------------------------------------------
    local function set_edit_pos(pattern,track,line,note_column,effect_column)
      
      local song = renoise.song()
      local pos = song.transport.edit_pos
      --default values
      pattern = pattern or song.selected_pattern_index
      track = track or  song.selected_track_index
      line = line or pos.line
      
      local note = false
      local effect = false
      
      if note_column and note_column ~= 0 then
        note = true
      elseif effect_column and effect_column ~= 0 then
        effect = true
      end
        
      --pos values (pattern in sequence and line)
      pos.sequence = pattern
      pos.line = line
      --update pos object
      song.transport.edit_pos = pos  
      
      --set the track
      song.selected_track_index = track
      
      --change column if column given--
      if note == true then 
        -- --check new note column is visible and show it if not
        if song.tracks[track].visible_note_columns < note_column then
          song.tracks[track].visible_note_columns = note_column
        end
        --set note column
        song.selected_note_column_index = note_column
      elseif effect == true then
        --check new effect note column is visible and show it if not
        if song.tracks[track].visible_effect_columns < effect_column then
          song.tracks[track].visible_effect_columns = effect_column
        end
        --set effect column
        song.selected_effect_column_index = effect_column
      end
     
     -- print("pattern: "..tostring(pattern))
     -- print("track: "..tostring(track))
     -- print("line: "..tostring(line))
     -- print("note column: "..tostring(note_column))
     -- print("effect column: "..tostring(effect_column))
      
    end
    
    --------------------------------------------------
    --get target positions
    local line = button + tool_offset
    local note_column = nil
    --get note or effect col
    local effect_column = nil
    if song.selected_note_column_index ~= 0 then
      note_column = song.selected_note_column_index
    else
      effect_column  = song.selected_effect_column_index
    end
    --------
    --set_edit_pos(pattern,track,line,note_column,effect_column) --or any as nil 
    set_edit_pos(nil,nil,line,note_column,effect_column)
    --reset global key_handler_value 
    g_keyhandler_value = nil
 

  
  do return end--]]
  
  

  
  --toggle beat markers and pattern notes and line numbers
  --[[
  --toggle note offs
  if options.transport_text_flag.value  == "beat markers" then --change now to see "pattern line numbers"
    options.transport_text_flag.value = "pattern line numbers"
    for i = 1,tool_range do
      vb.views["transport button"..i].text = tostring(i - 1)
    end
  elseif options.transport_text_flag.value == "pattern line numbers" then --change now to see "pattern notes"
    options.transport_text_flag.value = "pattern notes"
    --calculate beat markers (future proofing if tool range changes)
    
  elseif options.transport_text_flag.value == "pattern notes" then --change now to see "beat markers"
    --updat global beat_markers
    beat_markers = calculate_beat_markers()
    for i = 1,tool_range do
      vb.views["transport button"..i].text = beat_markers[i] 
    end
    options.transport_text_flag.value = "beat markers"  --this mode is taken care of in timer
  end--]]
end 


--value is the value of the slider, button is the slider index
----------main s--------------- 
------------------------------------- 
function main_sliders_notifier(value,button)  
-------------------------------------- 

  ----table to store all variables needed for sliders notifier "wiring" to the pattern editor
  --g_target = {}
  --g_target.column_index = 1
  --g_target.column_type = "note"
  --g_target.value_type = "volume"
  --g_target.all_note_columns = {}
  --g_target.all_fx_columns = {}
  
  --pattern variables
  local song = renoise.song() 
  local pattern = renoise.song().selected_pattern
  local pattern_length = pattern.number_of_lines
  local pattern_track = renoise.song().selected_pattern_track
   
  --if on fx track then return
  if song.tracks[song.selected_track_index].type ~= renoise.Track.TRACK_TYPE_SEQUENCER then
    status("Tool Only Works On Sequencer Tracks")
    return
  end
     
  --calculate the correct renoise pattern line index with global g_tool_start_line
  local target_pat_line = (button + g_tool_start_line -1)   
  local col = g_target.column_index 
  
  --get target tables to work with from global g_target{}
  --------------------------------------------------------
  --renoise note_column object
  --So 'note' stores a note column object.  If printed you get a string something like: "C-40002....0000"
  local note = nil
  
  --when starting on an fx, note cols may not yet be scanned
  if #g_target.all_note_columns > 0 then
    note = g_target.all_note_columns[col][target_pat_line] 
  end
  --similarly: effect column object (we need to check global table has been populated which it should be if user chose an fx button)
  --if not, it won't be needed in this run of function anyway
  local fx_col = nil
  if #g_target.all_fx_columns > 0 then 
    --get effect column object from global table g_target
    fx_col = g_target.all_fx_columns[g_slider_lane_two_col_target_index][target_pat_line]
  end
  
  
  -- NOTE + VOLUME SELECTED
  -------------------------
  if g_target.value_type == "note + volume" then
    
    --empty volume when slider at max
    if value == VEL_SLIDER_MAX then
      value = renoise.PatternLine.EMPTY_VOLUME --(EMPTY_VOL == 255)
    end
    
    note.volume_value = value
   
    --add a note if not there or a note-off is present
    --------------------------------------------------
    if note.note_string == "---" or note.note_string == "OFF" then 
      -----------------------
      --grab button visible (when button turns grey is this dealt with in timer???) --to check
      --if the grab button is visible
       ------------------------
      if (vb.views["grab extra"..button].visible == true) then
      
        --alternative currently disabled behavior: Toggle this button to grey and inactive.
        -----------------------------------------------------------------------------------
        --if vb.views["grab extra"..button].color[1] == COLOR_RED_FLAG then
        --  vb.views["grab extra"..button].color = COLOR_GREY
        --  vb.views["grab extra"..button].text = ""
        --  g_grab_extra_target_button = nil
        --end 
         
        --if grab button == RED
        if vb.views["grab button"].color[1] == COLOR_RED_FLAG then
          --get target note text from popup
          local text = vb.views["target note"].items[vb.views["target note"].value]
          --turn all other buttons grey
          for i = 1,tool_range do
            vb.views["grab extra"..i].color = COLOR_GREY
            vb.views["grab extra"..i].text = ""
          end
          --turn this button red
          vb.views["grab extra"..button].color = COLOR_RED
          --show note on 'this' sliders relevant button
          vb.views["grab extra"..button].text = text
          --update global
          g_grab_extra_target_button = button
        end 
      end --grab button visible
     
      --get note string from popup
      local text = vb.views["target note"].items[vb.views["target note"].value]
      --set string
      note.note_string = text
      --set volume from slider (notifier param)
      note.volume_value = value
      --set inst val. from selected instrument in renoise
      note.instrument_value = song.selected_instrument_index - LUA_COUNTS_FROM_1 
    end --end [note + volume selected]
 
  -- VOLUME SELECTED
  -------------------------
  elseif g_target.value_type == "volume" then
    --empty volume when slider at max
    if value == VEL_SLIDER_MAX then
      value = renoise.PatternLine.EMPTY_VOLUME --(EMPTY_VOL == 255)
    end
    --set volume value in pattern
    note.volume_value = value
     --add a note if not there
    if note.note_string == "---" or note.note_string == "OFF" then 
      --note.note_string = user_target_note
      note.volume_value = value
      --note.instrument_value = song.selected_instrument_index - LUA_COUNTS_FROM_1
   end  
   
  --OTHER TARGETS SELECTED
  ------------------------
  elseif g_target.value_type == "delay" then
    note.delay_value = value
  elseif g_target.value_type == "pitch" then
    note.note_value = value
    note.instrument_value = song.selected_instrument_index - 1
  elseif g_target.value_type == "pan" then
    note.panning_value = value
  elseif g_target.value_type == "0axy (Arpeggio)" then
    fx_col.number_string = "0A"
    fx_col.amount_value = value
  elseif g_target.value_type == "0uxx (Slide Pitch up)" then
    fx_col.number_string = "0U"
    fx_col.amount_value = value
  elseif g_target.value_type == "0dxx (Slide Pitch Down)" then
    fx_col.number_string = "0D"
    fx_col.amount_value = value
  elseif g_target.value_type == "0gxx (Glide Towards Note)" then
    fx_col.number_string = "0G"
    fx_col.amount_value = value
  elseif g_target.value_type == "0ixx (Fade Volume In)" then
    fx_col.number_string = "0I"
    fx_col.amount_value = value
  elseif g_target.value_type == "0oxx (Fade Volume Out)" then
    fx_col.number_string = "0O"
    fx_col.amount_value = value
  elseif g_target.value_type == "0txy (Tremolo)" then
    fx_col.number_string = "0T"
    fx_col.amount_value = value
  elseif g_target.value_type == "0cxy (Cut Note)" then 
    fx_col.number_string = "0C"
    fx_col.amount_value = value
  elseif g_target.value_type == "0sxx (Trigger Sample Slice / Offset)" then
    fx_col.number_string = "0S"
    fx_col.amount_value = value
  elseif g_target.value_type == "0bxx (Play Backwards)" then
    fx_col.number_string = "0B"
    if value > 0.5 then 
      value = 0   --reverse values for backwards command (makes more sense with sliders)
    else 
      value = 1
    end 
    fx_col.amount_value = value
  elseif g_target.value_type == "0vxy (Vibrato)" then
    fx_col.number_string = "0V"
    fx_col.amount_value = value
  elseif g_target.value_type == "0nxy (Auto Pan)" then
    fx_col.number_string = "0N"
    fx_col.amount_value = value
  elseif g_target.value_type == "0exx (Envelope Pos)" then
    fx_col.number_string = "0E"
    fx_col.amount_value = value
  elseif g_target.value_type == "DEVICE" then
    fx_col.number_string = g_user_effect_number_string
    fx_col.amount_value = value
  elseif g_target.value_type == "bypass" then 
   --do nothing
  end 
 
 
  -------------
  --CHORD MODE  
  -------------
  --When tool is in chord mode we copy the values freom the current note to all notes on the same line.
  
  --is chord mode enabled?
  if vb.views["chord mode"].color[1] ~= COLOR_GREY_FLAG then
   
     -----------------------------------------------------------
     local function chord_mode_copy_meta_values(target_pat_line)
     -----------------------------------------------------------
       local song = renoise.song()
       local pat_length = song.selected_pattern.number_of_lines
       local trk = song.selected_track
       local columns = song.selected_track.visible_note_columns
       local pat_trk = song.selected_pattern_track
       
       --get current line
       local cur_line = pat_trk:line(target_pat_line)
       --get selected column from global table
       local selected_col = g_target.column_index
       --there are two possible volume modes
       local vol_target = (g_target.value_type == "volume") or (g_target.value_type == "note + volume")

       --vol  
       -----
       if (trk.volume_column_visible == true) and vol_target then
         --loop cols and set all to the same as the selected column
         for col = 1,columns do
            --check note present
           if cur_line:note_column(col).note_value < 120 then --[120=Off, 121=Empty]
             --copy value
             cur_line:note_column(col).volume_value = cur_line:note_column(selected_col).volume_value
           end
         end
       end 
       --pan
       -----
       if (trk.panning_column_visible == true) and (g_target.value_type == "pan") then
         --loop cols and set all to the same as the selected column
         for col = 1,columns do
           --check note present
           if cur_line:note_column(col).note_value < 120 then
             cur_line:note_column(col).panning_value = cur_line:note_column(selected_col).panning_value
           end 
         end
       end  
       --delay
       -------
       if (trk.delay_column_visible == true) and (g_target.value_type == "delay") then
         --loop cols
         for col = 1,columns do
           --check note present
           if cur_line:note_column(col).note_value < 120 then
             cur_line:note_column(col).delay_value = cur_line:note_column(selected_col).delay_value
           end
         end 
       end  
     end--local fn
   
   --call local function to copy to all notes in current line
   chord_mode_copy_meta_values(target_pat_line) 
  
  end --if CHORD MODE
end --en fn
-----------
-----------
 
-------------------------------------------------- 
function note_name_textfield_notifier(text,button)
-------------------------------------------------- 
   
  if button == nil then
    return
  end
  --return if empty string
  if text == "" then
   return
  end
  
  bypass_timer = true
  --make text uppercase
  text = text:upper()
  --pattern variables
  local song = renoise.song() 
  local pattern = renoise.song().selected_pattern
  local pattern_length = pattern.number_of_lines
  local pattern_track = renoise.song().selected_pattern_track
  
  --if on fx track then return
  if song.tracks[song.selected_track_index].type ~= renoise.Track.TRACK_TYPE_SEQUENCER then
    status("Tool Only Works On Sequencer Tracks")
    bypass_timer = false
    return
  end
  
  local col = g_target.column_index
   --when an fx lane is chosen in renoise, target the notes from note column 1 which will be automatically showing 
  if g_target.column_type == "effect" then
    col = 1
  end
  
  local current_note = pattern_track:line(button+g_tool_start_line - 1 ):note_column(col)

  --if letter O is found then add a note-off
  if string.match(text, 'O') then
    current_note.note_string = "OFF"
    bypass_timer = false
    return
  end
  
  --if letter X is found then add a note-off 
  if text == 'X' then
    current_note.note_string = "---"
    current_note.instrument_string = ".."
    vb.views["note name textfield"..button].text = ""
    bypass_timer = false
    return
  end
                                        
  --match parts of the passed string and current note to overwrite
  --and set defaults if relevant characters not found --final default is C-4
  local note = string.match(text, '%a') or string.match(current_note.note_string, '%a') or string.match(user_target_note, '%a')
  local sharp = string.match(text, '#') or  string.match(text, '-') or string.match(current_note.note_string, '#') or "-"
  local octave = string.match(text, '%d') or string.match(current_note.note_string, '%d') or string.match(user_target_note,'%d')
  
  ----NOTE: The API seems to check when a '#` is not available i.e. b#3 and rounds it down to b-3, so no need to check that here----
  
  --check if there is an instreument index, if not then use selected inst in renoise
  if current_note.instrument_string == ".." then
    current_note.instrument_value = song.selected_instrument_index - LUA_COUNTS_FROM_1
  end

  local final_string = note..sharp..octave
  --add note to pattern
  current_note.note_string = final_string
  
  --update global target note to match the one just input by user
  --this way the next note added by a slider will be the same
  ---------------------------------------------------------------
  local popup_strings = vb.views["target note"].items
  --loop target-note popup (top row of gui) and match
  for i = 1,#popup_strings do
    if final_string == popup_strings[i] then
      vb.views["target note"].value = i
      break
    end
  end   
  --reset timer flag
  bypass_timer = false  
end

-------------------------------------             
function off_buttons_notifier(button)
-------------------------------------
  --bit hacky - chaining notifiers
  --use textfield note name notifier
  if vb.views["note name textfield"..button].text ~= "OFF" then
    vb.views["note name textfield"..button].text = "OFF"
    vb.views["off button"..button].color = COLOR_RED
  else --clear note/ note off
    vb.views["note name textfield"..button].text = "X"
    vb.views["off button"..button].color= COLOR_GREY
  end   
end

--------------------------------------
function fx_column_buttons_notifier(i)
--------------------------------------

  local song = renoise.song()
  
  --NEW Clear Pitch Vel Pan button colors 
  for j = 1,5 do  
    vb.views["note col target "..j].color = COLOR_GREY
  end
   
  --set global to target the correct column
   g_slider_lane_two_col_target_index = i --REMOVE???????
  
  local selected_popup_index = vb.views["popup col "..i].value
  
  --fire notifier function that sets up global tables for sliders
  if vb.views["popup col "..i].value ~= EMPTY_FX_COLUMN then
    g_target_notifier_tab_fx[selected_popup_index]()
    g_target.column_index = i
    g_target.column_type = "effect"
    --set column type flag to note
  else
    g_target.value_type = "bypassed"
  end  
          
  --make column visible if it is not
  if song.selected_track.visible_effect_columns < i then
    song.selected_track.visible_effect_columns = i
  end
  --select column in renoise
  song.selected_effect_column_index = i
  
  --re-focus pattern editor
  renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
  renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
  
  --create a selection table
  local select = {
    start_line = 1,
    start_track = song.selected_track_index,
    start_column = song.selected_effect_column_index + song.selected_track.visible_note_columns,--offset note cols
    end_line = song.selected_pattern.number_of_lines,
    end_track = song.selected_track_index,
    end_column =song.selected_effect_column_index + song.selected_track.visible_note_columns,--offset note cols
    }
  --update renoise selection
  song.selection_in_pattern = select
  
  --loop to change selected button colors
  for j = 1,8 do
    vb.views["Fx Column Button "..j].color = COLOR_GREY
  end
  --loop and clear note buttons(top row) to grey
   for j = 1,12 do
    vb.views["Note Column Button "..j].color = COLOR_GREY
  end
  --change `this` button to color orange
  vb.views["Fx Column Button "..i].color = COLOR_ORANGE
end

--[[
function fx_column_popups_notifier(value,i) 

  
 -- if value == #table_of_sample_effects and vb.views["Fx Column Button "..i].color[1] == COLOR_ORANGE_FLAG then --last in tble is empty value "---" value == EMPTY_FX_COLUMN
   -- g_target.value_type = "bypassed"
   -- return
--  end
  
  --select associated button if not selected
  fx_column_buttons_notifier(i)
  
  
  --fire notifier if relevant button for this column is selected
 -- if vb.views["Fx Column Button "..i].color[1] == COLOR_ORANGE_FLAG then
   -- g_target_notifier_tab_fx[value]()
   -- g_target.column_index = i
 -- end
end
 --]]
